
b.openHorizontalhideBox("");
b.closeBox();
b.openHorizontalBox("");

    b.create_small_rackknobr(PARAM("Bass"), "Bass");

    b.create_small_rackknobr(PARAM("Intensity"), "Treble");

    b.create_small_rackknobr(PARAM("Fuzz"), "Fuzz");
b.closeBox();
