#
# This is the GTK rc style file for default
#

include "common.rc"

style "gx_head_none"
{
  GxPaintBox::icon-set =1000
}

class "*GxPaintBox" style:highest "gx_head_none"

style "gx_head_blue_skin_icons"
{
  stock["bigknob"] = {{"tactile_knob_large_151f.png"}}
  stock["smallknob"] = {{"tactile_knob_small_99f.png"}}
  stock["smallknobr"] = {{"tactile_knob_medium_151f.png"}}
}

class "*GxKnob" style "gx_head_blue_skin_icons"

style "gx2_head_skin_knob_animation"
{

GxKnob::framecount = 0 # use 0 for auto (assumes square frames)
                            #  and >1 for non squareframe animations
}
class "*GxKnob" style "gx2_head_skin_knob_animation"

style "gx1_head_wheel_icons"
{
stock["wheel_back"] = {{"tactile_wheel_101f.png"}}

GxWheel::framecount = 101 # use 0 for auto (assumes square frames)
                            #  and >1 for non squareframe animations
}

class "*GxWheel" style "gx1_head_wheel_icons"

style "gx1_head_wheel_vertical_icons"
{
stock["wheel_vertical_back"] = {{"tactile_h_wheel_101f.png"}}

GxWheelVertical::framecount = 101 # use 0 for auto (assumes square frames)
                            #  and >1 for non squareframe animations
}

class "*GxWheelVertical" style "gx1_head_wheel_vertical_icons"

style "gx2_head_skin_switch_icons"
{
stock["switchit_on"] = {{"tactile_backlit_switchit_on.png"}}
stock["switchit_off"] = {{"tactile_backlit_switchit_off.png"}}
}


class "*GxToggleImage" style "gx2_head_skin_switch_icons"

