/* SKINS */
.mod-head-model-001{{{cns}}} {
    border-radius: 20px;
}
.mod-head-model-001-0550{{{cns}}} {
    background-image: url(/resources/heads/model-001/model-0550.png{{{ns}}});
    background-position: center center;
    background-repeat: no-repeat;
    font-family: Helvetica,Arial,Sans-serif;
    height: 259px;
    position: relative;
    width: 550px;
}

.mod-head-model-001-0700{{{cns}}} {
    background-image: url(/resources/heads/model-001/model-0700.png{{{ns}}});
    background-position: center center;
    background-repeat: no-repeat;
    font-family: Helvetica,Arial,Sans-serif;
    height: 260px;
    position: relative;
    width: 700px;
    border-radius: 18px;
}

.mod-head-model-001-0800{{{cns}}} {
    background-image: url(/resources/heads/model-001/model-0800.png{{{ns}}});
    background-position: center center;
    background-repeat: no-repeat;
    font-family: Helvetica,Arial,Sans-serif;
    height: 260px;
    position: relative;
    width: 800px;
}

.mod-head-model-001-0950{{{cns}}} {
    background-image: url(/resources/heads/model-001/model-0950.png{{{ns}}});
    background-position: center center;
    background-repeat: no-repeat;
    font-family: Helvetica,Arial,Sans-serif;
    height: 260px;
    position: relative;
    width: 950px;
}

/* THE AUTHOR */
.mod-head-model-001{{{cns}}} .mod-plugin-brand h1 {
    bottom: 90px;
    color: #fff;
    font-size: 14px;
    left: 54px;
    line-height: 1;
    margin: 0;
    position: absolute;
}

/* PLUGIN'S NAME */
.mod-head-model-001{{{cns}}} .mod-plugin-name h1 {
    bottom: 60px;
    color: #fff;
    font-size: 18px;
    left: 54px;
    line-height: 1;
    margin: 0;
    position: absolute;
}

/* CONTROLS */
.mod-head-model-001{{{cns}}} .mod-control-group {
    bottom: 149px;
    color: #000;
    font-size: 10px;
    height: 73px;
    left: 34px;
    position: absolute;
    right: 34px;
    top: 37px;
}

.mod-head-model-001{{{cns}}} .mod-control-group > div {
    margin-right: 2px;
}

/* SWITCH ON/OFF GENERAL */
.mod-head-model-001{{{cns}}} .mod-switch {
    cursor: pointer;
    float: left;
    height: 73px;
    position: relative;
    width: 90px;
}

.mod-head-model-001{{{cns}}} .mod-switch-image {
    background-image: url(/resources/switches/switch-001.png{{{ns}}});
    background-position: -9px 0px;
    background-repeat: no-repeat;
    background-size: auto 73px;
    height: 73px;
    width: 73px;
}

.mod-head-model-001{{{cns}}} .mod-switch-image.on {
    background-position: -82px 0px;
}

.mod-head-model-001{{{cns}}} .mod-switch-image.off {
    background-position: -9px 0px;
}

/* SWITCH ON/OFF - 0550 */
.mod-head-model-001-0550{{{cns}}} .mod-switch {
    width: 145px;
}

/* SWITCH ON/OFF - 0700 */
.mod-head-model-001-0700{{{cns}}} .mod-switch {
    width: 160px;
}

/* SWITCH ON/OFF - 0800 */
.mod-head-model-001-0800{{{cns}}} .mod-switch {
    width: 150px;
}

/* SWITCH ON/OFF - 0950 */
.mod-head-model-001-0950{{{cns}}} .mod-switch {
    width: 130px;
}


/* CHICKEN HEAD */
.mod-head-model-001{{{cns}}} .mod-chicken-head {
    float: left;
    height: 73px;
    overflow: hidden;
    position: relative;
    text-align: center;
    width: 80px;
}

.mod-head-model-001{{{cns}}} .mod-chicken-head .mod-chicken-head-image {
    background-image: url(/resources/knobs/chicken-head/_strip.png{{{ns}}});
    background-position: left 0px;
    background-repeat: no-repeat;
    background-size: auto 80px;
    height: 73px;
    margin-top: 3px;
    position: relative;
    width: 80px;
    z-index: 2;
}

.mod-head-model-001{{{cns}}} .mod-chicken-head .mod-chicken-head-title {
    font-family: "Helvetica Neue",Helvetica,Arial,sans-serif;
    font-size: 10px;
    font-weight: bold;
    left: 0;
    position: absolute;
    right: 0;
    text-transform: uppercase;
    top: 0;
    z-index: 1;
}

/* ENUMERATED LIST */
.mod-head-model-001{{{cns}}} .mod-enumerated {
    color: #000;
    display: inline-block;
    margin-top: 40px;
    overflow: hidden;
    position: absolute;
    right: 3px;
    top: 80px;
    width: 250px;
}

.mod-head-model-001{{{cns}}} .mod-enumerated .mod-enumerated-selected {
    background: rgba(255,255,255,.85);
    border-radius: 4px;
    box-shadow:inset 1px 1px 10px #000;
    font-size: 14px;
    height: 34px;
    line-height: 2.5;
    margin-right: 8px;
    overflow: hidden;
    padding: 0 9px;
}

/* WHEN IS HOVER */
.mod-head-model-001{{{cns}}} .mod-enumerated:hover .mod-enumerated-selected {
    border-radius: 4px 4px 0 0;
}

/* THE LIST OF OPTIONS */
.mod-head-model-001{{{cns}}} .mod-enumerated .mod-enumerated-list {
    background: rgba(255,255,255,.85);
    display: none;
    font-weight: bold;
    height: 69px;
    overflow: auto;
    position: relative;
    width: 242px;
    z-index: 3;
}

.mod-head-model-001{{{cns}}} .mod-enumerated .mod-enumerated-list > div {
    padding: 3px 9px;
}

.mod-head-model-001{{{cns}}} .mod-enumerated .mod-enumerated-list > div:hover {
    background: rgba(0,0,0,.35);
    cursor: pointer;
}

/* SEPARATOR */
.mod-head-model-001{{{cns}}} .mod-separator {
    background: #000;
    display: inline-block;
    float: left;
    height: 69px;
    margin-top: 2px;
    width: 1px;
}