b.openHorizontalhideBox("");
b.create_master_slider(PARAM("wet"), "Dry/Wet");
b.closeBox();

b.openHorizontalBox("");
b.create_small_rackknob(PARAM("freq"), "freq");
b.create_small_rackknob(PARAM("depth"), "depth");
b.create_small_rackknob(PARAM("width"), "width");
b.create_small_rackknob(PARAM("feedback"), "feedback");
b.create_small_rackknob(PARAM("mix"), "mix");
b.create_small_rackknob(PARAM("wet"), "wet");
b.closeBox();
