// generated from file '../src/faust/drumseq.dsp' by dsp2cc:
// Code generated with Faust 0.9.90 (http://faust.grame.fr)

#define FAUSTFLOAT float

namespace drumseq {

class Dsp {
private:
	int fSamplingFreq;
	int 	iVec0[2];
	double 	fConst0;
	double 	fConst1;
	double 	fConst2;
	FAUSTFLOAT 	fcheckbox0;
	FAUSTFLOAT	*fcheckbox0_;
	double 	fVec1[2];
	double 	fRec0[2];
	double 	fConst3;
	double 	fConst4;
	double 	fConst5;
	double 	fConst6;
	double 	fConst7;
	double 	fConst8;
	double 	fConst9;
	int 	iRec5[2];
	double 	fConst10;
	double 	fConst11;
	double 	fConst12;
	double 	fConst13;
	double 	fRec4[2];
	double 	fRec3[3];
	double 	fConst14;
	double 	fVec2[2];
	double 	fRec2[2];
	double 	fRec1[3];
	double 	fConst15;
	double 	fRec8[2];
	int 	iRec6[2];
	double 	fConst16;
	double 	fConst17;
	double 	fConst18;
	double 	fRec7[2];
	double 	fConst19;
	double 	fConst20;
	double 	fConst21;
	double 	fConst22;
	double 	fConst23;
	double 	fConst24;
	double 	fRec13[2];
	double 	fConst25;
	double 	fConst26;
	double 	fConst27;
	double 	fRec14[2];
	double 	fConst28;
	double 	fRec15[2];
	double 	fConst29;
	double 	fRec11[2];
	double 	fRec12[2];
	double 	fConst30;
	double 	fConst31;
	double 	fConst32;
	double 	fRec10[2];
	double 	fRec9[3];
	double 	fConst33;
	double 	fRec18[2];
	int 	iRec16[2];
	double 	fConst34;
	double 	fConst35;
	double 	fRec17[2];
	FAUSTFLOAT 	fslider0;
	double 	fRec19[2];
	FAUSTFLOAT 	fslider1;
	FAUSTFLOAT	*fslider1_;
	double 	fRec20[2];
	FAUSTFLOAT 	fcheckbox1;
	FAUSTFLOAT	*fcheckbox1_;
	double 	fVec3[2];
	double 	fRec23[2];
	int 	iRec21[2];
	double 	fConst36;
	double 	fRec22[2];
	FAUSTFLOAT 	fslider2;
	FAUSTFLOAT	*fslider2_;
	double 	fConst37;
	double 	fRec26[2];
	double 	fRec27[2];
	double 	fConst38;
	double 	fRec28[2];
	double 	fRec24[2];
	double 	fRec25[2];
	double 	fRec30[2];
	double 	fRec31[2];
	double 	fConst39;
	double 	fRec32[2];
	double 	fRec33[2];
	int 	iRec34[2];
	double 	fRec35[2];
	FAUSTFLOAT 	fcheckbox2;
	FAUSTFLOAT	*fcheckbox2_;
	double 	fVec4[2];
	double 	fRec36[2];
	double 	fConst40;
	double 	fRec41[2];
	double 	fVec5[2];
	int 	IOTA;
	double 	fVec6[64];
	double 	fConst41;
	int 	iConst42;
	int 	iConst43;
	double 	fConst44;
	double 	fConst45;
	double 	fConst46;
	double 	fConst47;
	double 	fConst48;
	double 	fConst49;
	double 	fRec42[2];
	double 	fConst50;
	double 	fRec43[2];
	double 	fConst51;
	double 	fRec39[2];
	double 	fRec40[2];
	double 	fConst52;
	double 	fConst53;
	double 	fConst54;
	double 	fRec46[2];
	double 	fConst55;
	double 	fRec47[2];
	double 	fConst56;
	double 	fRec44[2];
	double 	fRec45[2];
	double 	fConst57;
	double 	fConst58;
	double 	fConst59;
	double 	fRec50[2];
	double 	fConst60;
	double 	fRec51[2];
	double 	fConst61;
	double 	fRec48[2];
	double 	fRec49[2];
	double 	fConst62;
	double 	fConst63;
	double 	fConst64;
	double 	fRec54[2];
	double 	fConst65;
	double 	fRec55[2];
	double 	fConst66;
	double 	fRec52[2];
	double 	fRec53[2];
	double 	fVec7[2];
	double 	fRec38[2];
	double 	fRec37[3];
	double 	fRec58[2];
	int 	iRec56[2];
	double 	fRec57[2];
	int 	iRec59[2];
	double 	fConst67;
	double 	fRec60[2];
	double 	fConst68;
	double 	fRec62[2];
	double 	fConst69;
	double 	fRec61[3];
	FAUSTFLOAT 	fslider3;
	double 	fRec63[2];
	double 	fConst70;
	double 	fConst71;
	double 	fConst72;
	double 	fConst73;
	double 	fConst74;
	double 	fConst75;
	double 	fConst76;
	double 	fConst77;
	double 	fRec65[2];
	double 	fRec64[3];
	double 	fConst78;
	FAUSTFLOAT 	fcheckbox3;
	FAUSTFLOAT	*fcheckbox3_;
	double 	fVec8[2];
	double 	fRec68[2];
	int 	iRec66[2];
	double 	fConst79;
	double 	fConst80;
	double 	fConst81;
	double 	fRec67[2];
	double 	fConst82;
	double 	fConst83;
	double 	fConst84;
	double 	fConst85;
	double 	fConst86;
	double 	fConst87;
	double 	fConst88;
	double 	fConst89;
	double 	fConst90;
	double 	fConst91;
	double 	fRec70[2];
	double 	fRec69[3];
	double 	fConst92;
	int 	iRec71[2];
	double 	fConst93;
	double 	fRec72[2];
	int 	iRec73[2];
	double 	fConst94;
	double 	fRec74[2];
	double 	fConst95;
	double 	fConst96;
	double 	fConst97;
	double 	fRec75[2];
	double 	fConst98;
	double 	fRec76[2];
	int 	iRec77[2];
	double 	fConst99;
	double 	fRec78[2];
	double 	fConst100;
	double 	fConst101;
	double 	fConst102;
	double 	fRec79[2];
	double 	fConst103;
	double 	fRec80[2];
	FAUSTFLOAT 	fslider4;
	double 	fRec81[2];
	FAUSTFLOAT 	fslider5;
	double 	fRec82[2];


public:
	void clear_state_f();
	void init(unsigned int samplingFreq);
	void compute(int count, FAUSTFLOAT *input0, FAUSTFLOAT *output0);
	int register_par(const ParamReg& reg);
	Dsp();
	~Dsp();
};

} // end namespace drumseq
