
b.openHorizontalhideBox("");
    b.create_master_slider(PARAM("Drive"), 0);
b.closeBox();
b.openHorizontalBox("");

    b.create_small_rackknobr(PARAM("Drive"), 0);

    b.create_small_rackknobr(PARAM("Fuzz"), 0);

    b.create_small_rackknobr(PARAM("Input"), 0);

    b.create_small_rackknobr(PARAM("Volume"), 0);
b.closeBox();
