
b.openHorizontalhideBox("");
    b.create_master_slider(PARAM("Scream"), N_("Scream"));
b.closeBox();
b.openHorizontalBox("");

    b.create_small_rackknobr(PARAM("Scream"), N_("Scream"));
b.closeBox();
